<?php

namespace Status\Status\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Status\Status\Forms\Status as StatusForm;
use Status\Status\Http\Requests\StatusRequest;
use Status\Status\Interfaces\StatusRepositoryInterface;
use Status\Status\Repositories\Eloquent\Filters\StatusResourceFilter;
use Status\Status\Repositories\Eloquent\Presenters\StatusListPresenter;

/**
 * Resource controller class for status.
 */
class StatusResourceController extends BaseController
{

    /**
     * Initialize status resource controller.
     *
     *
     * @return null
     */
    public function __construct(StatusRepositoryInterface $status)
    {
        parent::__construct();
        $this->form = StatusForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('status.status.modules'), 'status', guard_url('status'));
        $this->repository = $status;
    }

    /**
     * Display a list of status.
     *
     * @return Response
     */
    public function index(StatusRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(StatusResourceFilter::class)
            ->setPresenter(StatusListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('status::status.names'))
            ->view('status::status.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display status.
     *
     * @param Request $request
     * @param Model   $status
     *
     * @return Response
     */
    public function show(StatusRequest $request, StatusRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('status::status.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('status::status.show')
            ->output();
    }

    /**
     * Show the form for creating a new status.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(StatusRequest $request, StatusRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('status::status.name'))
            ->view('status::status.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new status.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(StatusRequest $request, StatusRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('status::status.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('status/status/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/status/status'))
                ->redirect();
        }

    }

    /**
     * Show status for editing.
     *
     * @param Request $request
     * @param Model   $status
     *
     * @return Response
     */
    public function edit(StatusRequest $request, StatusRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('status::status.name'))
            ->view('status::status.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the status.
     *
     * @param Request $request
     * @param Model   $status
     *
     * @return Response
     */
    public function update(StatusRequest $request, StatusRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('status::status.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('status/status/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('status/status/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the status.
     *
     * @param Model   $status
     *
     * @return Response
     */
    public function destroy(StatusRequest $request, StatusRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('status::status.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('status/status/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('status/status/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
